
// Includes definitions relevant to the peripherals embedded in the digital signal controller.
#include "F2806x_Device.h"

// Includes the definitions of the constants used in the program
#include "constants.h"


extern interrupt void Timer0_isr(void);

// Functions related the the setup of the hardware when using the BoosterBoard_01


int Enable_the_Interrupts(void){
    int Result=CORRECT;

    EALLOW;
    DINT;   // Disable all the interrupts
    IER=0x0000;
    IFR=0x0000;

    PieVectTable.TINT0=&Timer0_isr;
    PieCtrlRegs.PIECTRL.bit.ENPIE=1;
    PieCtrlRegs.PIEIER1.bit.INTx7=1;

    IER|=M_INT1;
    EINT;

    EDIS;

    return Result;
}



int Initialize_the_Clocks(void){
    int Result=CORRECT;

    if(SysCtrlRegs.PLLSTS.bit.MCLKSTS==1){
        Result=ERROR;
    }
    else{

        EALLOW;

// Initializes the clock generation
        SysCtrlRegs.XCLK.bit.XCLKOUTDIV=3;      // Disables the clock output pin
        SysCtrlRegs.CLKCTL.bit.XCLKINOFF=1;     // Disables the input for the external clock
        SysCtrlRegs.CLKCTL.bit.INTOSC2OFF=1;    // Disables the internal oscillator 2

// Initialize the PLL (see pag. 82 of the TMS320x2806x Technical Reference Manual)
        SysCtrlRegs.PLLSTS.bit.DIVSEL=0;        // Sets the CPU clock frequency equal to the oscillator clock frequency/4
        SysCtrlRegs.PLLSTS.bit.MCLKOFF=1;       // Disable failed oscillator detect logic
        SysCtrlRegs.PLLCR.bit.DIV=8;            // Sets the PLL output frequency to 8*oscillator lock frequency

        while(SysCtrlRegs.PLLSTS.bit.PLLLOCKS==0){
                                                // Waits for the PLL to lock
        }

        SysCtrlRegs.PLLSTS.bit.MCLKOFF=0;       // Enables failed oscillator detect logic

        SysCtrlRegs.PLLSTS.bit.DIVSEL=3;        // Sets the CPU clock frequency equal to the oscillator clock frequency
                                     // In this case is 80 MHz. The maximum is 90 MHz

// Sets the peripheral clock frequency
        SysCtrlRegs.LOSPCP.bit.LSPCLK=2;        // Peripheral Low Speed Clock = System Clock/4

        EDIS;
    }

    return Result;
}



int Enable_the_Peripherals(void){
    int Result=CORRECT;

    EALLOW;

    SysCtrlRegs.PCLKCR3.bit.CPUTIMER0ENCLK=1;
    SysCtrlRegs.PCLKCR0.bit.ADCENCLK=1;
    SysCtrlRegs.PCLKCR1.bit.EPWM4ENCLK=1;
    SysCtrlRegs.PCLKCR1.bit.EPWM5ENCLK=1;
    SysCtrlRegs.PCLKCR1.bit.EPWM6ENCLK=1;

    EDIS;


    return Result;
}



int Initialize_the_Timer0(int Number_of_Tenths_of_Millisecond){
    int Result=CORRECT;

    EALLOW;

    CpuTimer0Regs.TCR.bit.TSS=1; // Stops the timer


    CpuTimer0Regs.TPR.bit.TDDR=0x3f;
    CpuTimer0Regs.TPRH.bit.TDDRH=0x1f;

//    CpuTimer0Regs.TPRH.bit.TDDRH=0;
//    CpuTimer0Regs.TPR.bit.TDDR=799;

    CpuTimer0Regs.PRD.half.LSW=Number_of_Tenths_of_Millisecond;

    CpuTimer0Regs.TCR.bit.TRB=1;
    CpuTimer0Regs.TCR.bit.TIE=1;
    CpuTimer0Regs.TCR.bit.TIF=1;


    EDIS;


    return Result;
}


int Initialize_the_ADC(void){
    int Result=CORRECT;
    int Attesa=1000;

    EALLOW;

    AdcRegs.ADCCTL1.bit.ADCREFSEL=0;    // Selects the internal voltage reference
    AdcRegs.ADCCTL1.bit.ADCPWDN=1;
    AdcRegs.ADCCTL1.bit.ADCBGPWD=1;
    AdcRegs.ADCCTL1.bit.ADCREFPWD=1;
    AdcRegs.ADCCTL1.bit.ADCENABLE=1;

    while(Attesa-->0){};

    AdcRegs.ADCCTL2.bit.ADCNONOVERLAP=1;
    AdcRegs.ADCCTL2.bit.CLKDIV2EN=1;
    AdcRegs.ADCCTL2.bit.CLKDIV4EN=1;

    AdcRegs.ADCINTFLGCLR.all=0xFFFF;


    AdcRegs.ADCCTL1.bit.INTPULSEPOS=1;
    AdcRegs.ADCCTL1.bit.TEMPCONV=1;

    AdcRegs.INTSEL1N2.bit.INT1CONT=0;
    AdcRegs.INTSEL1N2.bit.INT1E=0;                                              // Mettere a 1 per interrupt
    AdcRegs.INTSEL1N2.bit.INT1SEL=3;

    AdcRegs.INTSEL1N2.bit.INT2CONT=0;
    AdcRegs.INTSEL1N2.bit.INT2E=0;                                              // Mettere a 1 per interrupt
    AdcRegs.INTSEL1N2.bit.INT2SEL=5;

    AdcRegs.ADCSOC3CTL.bit.TRIGSEL=0;                                           // Cambiare per comando da PWM
    AdcRegs.ADCSOC3CTL.bit.CHSEL=6;
    AdcRegs.ADCSOC3CTL.bit.ACQPS=0x3F;

    AdcRegs.ADCSOC5CTL.bit.TRIGSEL=0;                                           // Cambiare per comando da PWM
    AdcRegs.ADCSOC5CTL.bit.CHSEL=5;
    AdcRegs.ADCSOC5CTL.bit.ACQPS=0x3F;



    EDIS;


    return Result;
}


int Initialize_the_PWM(void){
    int Result=CORRECT;

    EALLOW;
    EPwm4Regs.TBCTL.bit.CLKDIV=0;
    EPwm4Regs.TBCTL.bit.HSPCLKDIV=0;
    EPwm4Regs.TBCTL.bit.CTRMODE=3;

    EPwm4Regs.TBPRD=4000;





    return Result;
}



int Sample_the_Analog_Voltage(){
    AdcRegs.ADCSOCFRC1.bit.SOC3=1;
    while(AdcRegs.ADCINTFLG.bit.ADCINT1==1){}
    AdcRegs.ADCINTFLGCLR.bit.ADCINT1=1;
    return AdcResult.ADCRESULT3;

}


int Sample_the_Temperature(){
    AdcRegs.ADCSOCFRC1.bit.SOC5=1;
    while(AdcRegs.ADCINTFLG.bit.ADCINT2==1){}
    AdcRegs.ADCINTFLGCLR.bit.ADCINT2=1;
    return AdcResult.ADCRESULT5;

}


int Initialize_the_GPIO(void){
    int Result=CORRECT;
// GPIO 34 and GPIO 39 are connected to the LAUNCHXL LEDs
    EALLOW;

    GpioCtrlRegs.GPBMUX1.bit.GPIO34=0;          // Sets GPIO34 as general purposes digital I/O
    GpioCtrlRegs.GPBMUX1.bit.GPIO39=0;          // Sets GPIO39 as general purposes digital I/O

    GpioCtrlRegs.GPBDIR.bit.GPIO34=1;           // Configures GPIO34 as output
    GpioCtrlRegs.GPBDIR.bit.GPIO39=1;           // Configures GPIO39 as output

    GpioDataRegs.GPBSET.bit.GPIO34=1;           // Sets pin GPIO34
    GpioDataRegs.GPBSET.bit.GPIO39=1;           // Sets pin GPIO39

// Pushbutton
    GpioCtrlRegs.GPBMUX2.bit.GPIO52=0;          // Sets GPIO52 as general purposes digital I/O

    GpioCtrlRegs.GPBDIR.bit.GPIO52=0;           // Configures GPIO52 as input

// Three colors LED
    GpioCtrlRegs.GPAMUX1.bit.GPIO6=0;           // Sets GPIO06 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO8=0;           // Sets GPIO08 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO10=0;          // Sets GPIO10 as general purposes digital I/O


    GpioCtrlRegs.GPADIR.bit.GPIO6=1;            // Configures GPIO06 as output
    GpioCtrlRegs.GPADIR.bit.GPIO8=1;            // Configures GPIO08 as output
    GpioCtrlRegs.GPADIR.bit.GPIO10=1;           // Configures GPIO10 as output

    GpioDataRegs.GPACLEAR.bit.GPIO6=1;          // Clears pin GPIO06    (Red)
    GpioDataRegs.GPACLEAR.bit.GPIO8=1;          // Clears pin GPIO08    (Blue)
    GpioDataRegs.GPACLEAR.bit.GPIO10=1;         // Clears pin GPIO010   (Green)

    // Set up of the GPIO pins for the management of the 7 segments display

    // GPIOs connected to the four digit display
    // GPIO 00 -> Enables digit 4
    // GPIO 01 -> Enables digit 3
    // GPIO 02 -> Enables digit 2
    // GPIO 03 -> Enables digit 1                           ___A_____
    // GPIO 04 -> Enables segment F                       /          /
    // GPIO 05 -> Enables segment D                       /          /
    // GPIO 13 -> Enables segment C                     F /          / B
    // GPIO 19 -> Enables segment B                       /____G_____/      dp
    // GPIO 44 -> Enables segment G                       /          /     
    // GPIO 16 -> Enables segment dp                    E /          / C
    // GPIO 17 -> Enables segment E                       /          /
    // GPIO 50 -> Enables segment A                       /____D_____/

    GpioCtrlRegs.GPAMUX1.bit.GPIO0=0;           // Sets GPIO00 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO1=0;           // Sets GPIO01 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO2=0;           // Sets GPIO02 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO3=0;           // Sets GPIO03 as general purposes digital I/O

    GpioCtrlRegs.GPADIR.bit.GPIO0=1;            // Configures GPIO00 as output
    GpioCtrlRegs.GPADIR.bit.GPIO1=1;            // Configures GPIO01 as output
    GpioCtrlRegs.GPADIR.bit.GPIO2=1;            // Configures GPIO02 as output
    GpioCtrlRegs.GPADIR.bit.GPIO3=1;            // Configures GPIO03 as output


    GpioCtrlRegs.GPAMUX1.bit.GPIO4=0;           // Sets GPIO04 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO5=0;           // Sets GPIO05 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX1.bit.GPIO13=0;          // Sets GPIO13 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX2.bit.GPIO16=0;          // Sets GPIO016 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX2.bit.GPIO17=0;          // Sets GPIO016 as general purposes digital I/O
    GpioCtrlRegs.GPAMUX2.bit.GPIO19=0;          // Sets GPIO016 as general purposes digital I/O
    GpioCtrlRegs.GPBMUX1.bit.GPIO44=0;          // Sets GPIO44 as general purposes digital I/O
    GpioCtrlRegs.GPBMUX2.bit.GPIO50=0;          // Sets GPIO50 as general purposes digital I/O

    GpioCtrlRegs.GPADIR.bit.GPIO4=1;            // Configures GPIO04 as output
    GpioCtrlRegs.GPADIR.bit.GPIO5=1;            // Configures GPIO05 as output
    GpioCtrlRegs.GPADIR.bit.GPIO13=1;           // Configures GPIO13 as output
    GpioCtrlRegs.GPADIR.bit.GPIO16=1;           // Configures GPIO16 as output
    GpioCtrlRegs.GPADIR.bit.GPIO17=1;           // Configures GPIO17 as output
    GpioCtrlRegs.GPADIR.bit.GPIO19=1;           // Configures GPIO19 as output
    GpioCtrlRegs.GPBDIR.bit.GPIO44=1;           // Configures GPIO44 as output
    GpioCtrlRegs.GPBDIR.bit.GPIO50=1;           // Configures GPIO50 as output


    EDIS;

    return Result;
}


int Manage_LEDs(int action, int color){
    int Result=CORRECT;

    if(action==TURN_ON){
        if(color==RED){
            GpioDataRegs.GPBCLEAR.bit.GPIO34=1;
        }
        else{
            if(color==BLUE){
                GpioDataRegs.GPBCLEAR.bit.GPIO39=1;
            }
            else{
                Result=ERROR;
            }
        }
    }
    else{
        if(action==TURN_OFF){
            if(color==RED){
                GpioDataRegs.GPBSET.bit.GPIO34=1;
            }
            else{
                if(color==BLUE){
                    GpioDataRegs.GPBSET.bit.GPIO39=1;
                }
                else{
                    Result=ERROR;
                }
            }
        }
        else{
            Result=ERROR;
        }

    }

    return Result;
}


int Show_Character_on_Digit(char Character, int Digit){
    int Result=CORRECT;


    // Set up of the GPIO pins for the management of the 7 segments display

    // GPIOs connected to the four digit display
    // GPIO 00 -> Enables digit 4
    // GPIO 01 -> Enables digit 3
    // GPIO 02 -> Enables digit 2
    // GPIO 03 -> Enables digit 1                           ___A_____
    // GPIO 04 -> Enables segment F                       /          /
    // GPIO 05 -> Enables segment D                       /          /
    // GPIO 13 -> Enables segment C                     F /          / B
    // GPIO 19 -> Enables segment B                       /____G_____/      dp
    // GPIO 44 -> Enables segment G                       /          /     
    // GPIO 16 -> Enables segment dp                    E /          / C
    // GPIO 17 -> Enables segment E                       /          /
    // GPIO 50 -> Enables segment A                       /____D_____/

    GpioDataRegs.GPACLEAR.all=0x0000000F;   // Clears pins GPIO00 to GPIO03 and turns off all the digits
    GpioDataRegs.GPACLEAR.all=0x000B2030;   // Clears all the segments
    GpioDataRegs.GPBCLEAR.all=0x00041000;   // Clears all the segments

    switch(Character){

        case '0':
            GpioDataRegs.GPASET.all = 0x000A2030;
            GpioDataRegs.GPBSET.all = 0x00040000;
            break;

        case '1':
            GpioDataRegs.GPASET.all = 0x00082000;
            break;

        case '2':
            GpioDataRegs.GPASET.all = 0x000A0020;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '3':
            GpioDataRegs.GPASET.all = 0x00082020;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '4':
            GpioDataRegs.GPASET.all = 0x00082010;
            GpioDataRegs.GPBSET.all = 0x00001000;
            break;

        case '5':
            GpioDataRegs.GPASET.all = 0x00002030;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '6':
            GpioDataRegs.GPASET.all = 0x00022030;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '7':
            GpioDataRegs.GPASET.all = 0x00082000;
            GpioDataRegs.GPBSET.all = 0x00040000;
            break;

        case '8':
            GpioDataRegs.GPASET.all = 0x000A2030;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '9':
            GpioDataRegs.GPASET.all = 0x00082030;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case '-':
            GpioDataRegs.GPBSET.all = 0x00001000;
            break;

        case 'E':
            GpioDataRegs.GPASET.all = 0x00020030;
            GpioDataRegs.GPBSET.all = 0x00041000;
            break;

        case 'r':
            GpioDataRegs.GPASET.all = 0x00020000;
            GpioDataRegs.GPBSET.all = 0x00001000;
            break;

        default:
            Result = ERROR;
            break;

    }


    switch(Digit){
        case 1:
            GpioDataRegs.GPASET.bit.GPIO3=1;              // Sets pin GPIO03 // Turns on Digit 1
            break;

        case 2:
            GpioDataRegs.GPASET.bit.GPIO2=1;              // Sets pin GPIO03 // Turns on Digit 1
            break;

        case 3:
            GpioDataRegs.GPASET.bit.GPIO1=1;              // Sets pin GPIO03 // Turns on Digit 1
            break;

        case 4:
            GpioDataRegs.GPASET.bit.GPIO0=1;              // Sets pin GPIO03 // Turns on Digit 1
            break;

        default:
            Result=ERROR;
            break;
    }

    return Result;

}

